#include "stdafx.h"

void Histogram(IplImage * src, IplImage ** dst, IplImage * mask, CString title,int color)
{
	// src   = immagine da analizzare (deve essere gi ad un solo canale)
	// dst   = immagine che conterr l'istogramma
	// mask  = immagine maschera
	// title = titolo dell'istogramma
	// color = colore delle barre e del titolo
	
	const int BINS=256;
	int dims[1]={BINS};					// numero di barre per dimensione
	float thresh[1][2] = { {0, 255} };	// range di valori da analizzare
	float *pthresh[1] = { thresh[0] };

	CvHistogram* hist = cvCreateHist(1, dims, CV_HIST_ARRAY,0, 1);
	cvSetHistBinRanges( hist, pthresh , 1);

	cvCalcHist(&src, hist, 0, mask );

	// crea l'immagine per l'istogramma
	// (eventualmente rilascia l'immagine inizialmente presente in dst)
	if(*dst!=NULL) cvReleaseImage(dst);
	*dst=cvCreateImage(cvSize(BINS*2+64,432),IPL_DEPTH_8U,3);

	// sfondo bianco per l'istogramma
	cvSet(*dst,cvScalar(0,0,0));
	
	// traccio gli assi in grigio
	int gray=CV_RGB(170,170,170);
	cvLine(*dst,cvPoint(48,416),cvPoint(48+BINS*2,416),gray);
	cvLine(*dst,cvPoint(48,416),cvPoint(48,16),gray);
	cvLine(*dst,cvPoint(50,417),cvPoint(50,417),gray);
	cvLine(*dst,cvPoint(50+2*(BINS/4-1),417),cvPoint(50+2*(BINS/4-1),417),gray);
	cvLine(*dst,cvPoint(50+2*(BINS/2-1),417),cvPoint(50+2*(BINS/2-1),417),gray);
	cvLine(*dst,cvPoint(50+2*(3*BINS/4-1),417),cvPoint(50+2*(3*BINS/4-1),417),gray);
	cvLine(*dst,cvPoint(50+2*(BINS-1),417),cvPoint(50+2*(BINS-1),417),gray);
	cvLine(*dst,cvPoint(47,416),cvPoint(47,416),gray);
	cvLine(*dst,cvPoint(47,16),cvPoint(47,16),gray);
	cvLine(*dst,cvPoint(47,116),cvPoint(47,116),gray);
	cvLine(*dst,cvPoint(47,216),cvPoint(47,216),gray);
	cvLine(*dst,cvPoint(47,316),cvPoint(47,316),gray);
	
	// traccio il titolo dell'istogramma
	CvFont font;
	cvInitFont(&font,CV_FONT_VECTOR0,0.5,0.3,0,1);
	cvPutText(*dst,title,cvPoint(150,9),&font,color);
	
	// calcolo il massimo per la normalizzazione dell'istogramma
	float minVal[1];
	float maxVal[1];
	cvGetMinMaxHistValue (hist,minVal,maxVal);

	// traccio le etichette sugli assi
	cvInitFont(&font,CV_FONT_VECTOR0,0.3,0.3,0,1);
	char buffer[11];
	cvPutText(*dst,_gcvt(maxVal[0],10,buffer),cvPoint(3,20),&font,gray);
	cvPutText(*dst,_gcvt(3*maxVal[0]/4,10,buffer),cvPoint(3,120),&font,gray);
	cvPutText(*dst,_gcvt(maxVal[0]/2,10,buffer),cvPoint(3,220),&font,gray);
	cvPutText(*dst,_gcvt(maxVal[0]/4,10,buffer),cvPoint(3,320),&font,gray);
	cvPutText(*dst,"0",cvPoint(35,420),&font,gray);
	cvPutText(*dst,"0",cvPoint(48,427),&font,gray);
	cvPutText(*dst,_itoa(BINS/4-1,buffer,10),cvPoint(44+2*(BINS/4-1),427),&font,gray);
	cvPutText(*dst,_itoa(BINS/2-1,buffer,10),cvPoint(40+2*(BINS/2-1),427),&font,gray);
	cvPutText(*dst,_itoa(3*BINS/4-1,buffer,10),cvPoint(40+2*(3*BINS/4-1),427),&font,gray);
	cvPutText(*dst,_itoa(BINS-1,buffer,10),cvPoint(40+2*(BINS-1),427),&font,gray);

	// tracciamento dell'istogramma
	float bin;
	int valore;
	for(int i=0; i<BINS; i++)
	{
		bin=cvQueryHistValue_1D(hist,i);
		valore=int(400*(bin/maxVal[0]));
		if(valore)
			cvLine(*dst,cvPoint(50+2*i,415),cvPoint(50+2*i,416-valore),color);
	}
}


HINSTANCE DLLhinst;

int OpenImage(IplImage ** dst,int iscolor)
{		
	// dst = immagine destinazione
	// iscolor = parametro da passare a cvLoadImage:
	//	 >0  l'immagine viene aperta con tre canali
	//   =0        "       "     "   con un solo canale
	//   <0        "       "     "   cos com'
	
	IplImage* temp;

	char path[256];
	//Inizializzazione di path
	memset(path,0,256);
	//scelgo il file da aprire attraverso la finestra open-file
	OPENFILENAME fn;
	fn.lStructSize = sizeof (OPENFILENAME);
	fn.hwndOwner = NULL;
	fn.lpstrFilter = "Immagini (*.bmp;*.jpg;*.tif)\0*.bmp;*.jpg;*.tif\0Tutti i file(*.*)\0*.*\0\0";
	fn.lpstrFile = path;
	fn.nMaxFile = 256;
	fn.lpstrFileTitle = NULL;
	fn.lpstrInitialDir = NULL;
	fn.lpstrTitle = NULL;
	fn.Flags = NULL;
	fn.lpstrDefExt = "bmp";
	fn.hInstance = DLLhinst;
	fn.lpfnHook = NULL;
	fn.lpstrCustomFilter = NULL;
	fn.lCustData = NULL;
	
	if(!GetOpenFileName(&fn)) return 0;  // ritorna 0 se l'utente ha annullato
	temp=cvLoadImage(path,iscolor);
	if(temp!=NULL) {
		if(*dst!=NULL)
			cvReleaseImage(dst);	// rilascia l'immagine precedentemente contenuta in dst
		*dst=temp;
		return 1;	// ritorna 1 se l'immagine  stata aperta correttamente
	}
	else 
		return -1;  // ritorna -1 se c' stato un errore nel caricamento dell'immagine
					// (ad esempio l'immagine non  valida)
}